/* -----------------------------------------------------------------
    Weather Shield 1 communication sample sketch
                               www.EtherMania.com - Signorini Marco
   
   This is a sample sketch that shows how to communicate to the
   Weather Shield and an Arduino 2009 board and to publish 
   received to a pachube feed.
   This is a modification of a pachube provided sample.
  ------------------------------------------------------------------- */
  
#include <WeatherShield1.h>
#include <SPI.h>
#include <Ethernet.h>
#include <string.h>
#include "weathershield1_weatherdata.h"

#undef int() // needed by arduino 0011 to allow use of stdio
#include <stdio.h> // for function sprintf

#define SHARE_FEED_ID               10299     // this is your Pachube feed ID that you want to share to
#define REMOTE_FEED_ID             10299   // this is the ID of the remote Pachube feed that you want to connect to
#define REMOTE_FEED_DATASTREAMS    1        // make sure that remoteSensor array is big enough to fit all the remote data streams
#define UPDATE_INTERVAL            10000    // if the connection is good wait 30 seconds before updating again - should not be less than 5
#define RESET_INTERVAL             10000    // if connection fails/resets wait 10 seconds before trying again - should not be less than 5
#define SAMPLEINTERVAL             10000    // sample weather data each x milliseconds

#define PACHUBE_API_KEY            "3e8685d99881098340c059842645739289522324472aab2" // fill in your API key 
#define RXBUFFERLENGTH          4


byte mac[] = { 0xCC, 0xAC, 0xBE, 0xEF, 0xFE, 0x95 }; 	// make sure this is unique on your network
byte ip[] = { 192, 168, 0, 150   };                  		// no DHCP so we set our own IP address
byte remoteServer[] = { 209,40,205,190 };            		// pachube.com

float remoteSensor[REMOTE_FEED_DATASTREAMS];        // we know that feed has floats - this might need changing for feeds without floats
unsigned long lastSample;

WeatherShield1 weatherShield;
WeatherData weatherData;

/* This is the prototype function able to communicate with the WeatherShield1 */
boolean handleWeatherShieldValues(WeatherData &weatherData);
void validateBufferData(WeatherData &weatherData);

void setup()
{
  Serial.begin(9600); 
  setupEthernet(); 
  lastSample = millis() - (SAMPLEINTERVAL + 1);
}

void loop()
{

  // Read info from the weather shield
  if ((millis() - lastSample) > SAMPLEINTERVAL) {
    lastSample = millis();
    
    Serial.println("Data sampled from Weather Shield");
    
    handleWeatherShieldValues(weatherData);
    validateBufferData(weatherData);
  }
  
  // call 'pachube_in_out' at the beginning of the loop, handles timing, requesting
  // and reading. use serial monitor to view debug messages
  pachube_in_out();
}


/* ----------------------------------------------------------------- */

/* This function will read weather shield values and store in the
provided container. It returns true if the values are ready to be used, 
false if something goes wrong with the WeatherShield1 */
boolean handleWeatherShieldValues(WeatherData &weatherData) {
  
  /* This is the buffer for the answers */
  unsigned char ucBuffer[RXBUFFERLENGTH];
  
  weatherData.reset();
    
  /* Check for the weather shield connection and read values */
  if (weatherShield.sendCommand(CMD_ECHO_PAR, 100, ucBuffer)) {
    
    /* Start reading temperature */
    if (weatherShield.sendCommand(CMD_GETTEMP_C_AVG, 0, ucBuffer)) {
       weatherShield.decodeFloatAsString(ucBuffer, weatherData.chTemperature);
    }
           
       
    /* Read pressure values */
    if (weatherShield.sendCommand(CMD_GETPRESS_AVG, 0, ucBuffer)) {
       weatherShield.decodeFloatAsString(ucBuffer, weatherData.chPressure);
    }

    
    /* Read humidity values */
    if (weatherShield.sendCommand(CMD_GETHUM_AVG, 0, ucBuffer)) {
       weatherShield.decodeFloatAsString(ucBuffer, weatherData.chHumidity);
    }
             
    return true;
  }  
  
  weatherData.bReady = false;
  
  return false;
}

// We would like to send data to pachube only when the moving average calculated by the 
// WeatherShield contains a valid value (i.e. all eight samples were properly collected)
void validateBufferData(WeatherData &weatherData) {
  
    unsigned char ucN;
    unsigned char ucBuffer[RXBUFFERLENGTH];
    
    boolean valid = true;
    for (ucN = 0; ucN < 8; ucN++) {
      
      unsigned short value = 0;
      if (weatherShield.sendCommand(CMD_GETTEMP_C_RAW, ucN, ucBuffer))
        value = weatherShield.decodeShortValue(ucBuffer);
        
      valid &= (value != 0);
    }
    
    // Set the data ready flag
    weatherData.bReady = valid;
}

